import styled, {css} from 'styled-components';
import {fromRegistry, registry} from '../../core';
import {IColors, ITheme} from '../..';
import {ButtonHTMLAttributes} from 'react';

const getColorByProps = ({theme, primary, danger, success}) => {
  switch (true) {
    case primary:
      return theme.color.primary;
    case danger:
      return theme.color.danger;
    case success:
      return theme.color.success;
    default:
      return theme.color.secondary;
  }
};

const getTextColorByProps = ({theme, primary, danger, success}) => {
  switch (true) {
    case primary:
    case danger:
    case success:
      return theme.color.background;
    default:
      return theme.color.text;
  }
};

export interface IButton extends ButtonHTMLAttributes<HTMLButtonElement> {
  fluid?: boolean;
  theme?: ITheme;
  primary?: IColors | string;
  danger?: IColors | string;
  success?: IColors | string;
}

registry.register('Button', styled.button`
    display: ${({fluid}) => fluid ? 'block' : 'inline-block'};
    width: ${({fluid}) => fluid ? '100%' : 'auto'};
    background-color: ${(props) => getColorByProps(props).toString()};
    color: ${(props) => getTextColorByProps(props)};
    outline: none;
    cursor: pointer;
    font-size: ${({theme}) => theme.font.control};
    line-height: 1;
    padding: ${({theme}) => theme.padding.control};
    border-radius: 4px;
    transition: all .15s ease-in-out;
    border: ${({theme}) => theme.border.control};
    box-sizing: border-box;
    box-shadow: ${({theme}) => theme.shadow.control};

    &:hover {
      background-color: rgba(0,0,0,.04);
    }

    &:active {
      background-color: rgba(0,0,0,.08);
      box-shadow: none;
    }

    ${(props) => !props.disabled ? activeCss(props) : disabledCss(props)}
`);

const activeCss = (props) => css`
  &:hover {
    background-color: ${getColorByProps(props).hover};
  }
  &:active {
    background-color: ${getColorByProps(props).active};
  }
`;

const disabledCss = (props) => css`
  cursor: default;
  background-color: ${getColorByProps(props).disabled};
  &:hover {
    background-color: ${getColorByProps(props).disabled};
  }
`;

export const Button = fromRegistry<IButton>('Button');
