import React, {Component} from 'react';
import styled, {css} from 'styled-components';
import {Row, ThemeConsumer} from '../..';
import {Svg, Icon} from '../Icon/Icon';
import tick from '!svgson-loader!./tick.svg';
import remove from '!svgson-loader!./remove.svg';

const Input = styled.input.attrs({type: 'checkbox'})`
  width: auto;
  height: ${({theme}) => theme.size};
  margin: 0;
  padding: 0;
  opacity: 0;
  position: absolute;
  cursor: pointer;
`;

const Label = styled.label`
  font-family: ${({theme}) => theme.font.family};
  font-size: ${({theme}) => theme.font.element};
  line-height: 20px;
  font-weight: 400;
  cursor: pointer;
  ${({checked, theme}) => !checked ?
  css`
      &:hover ${Box} {
        background-color: ${theme.color.backgroundSecondary};
      }

      &:hover ${Svg} {
        fill: ${theme.color.secondaryIcon};
      }
    `
  : null
  }
`;

const Box = styled.div`
  height: ${({theme}) => theme.size};
  width: ${({theme}) => theme.size};
  position: relative;
  border: ${({theme, checked}) => `1px solid ${checked ? theme.color.primary : theme.color.border}`};
  border-radius: ${({theme}) => theme.border.radius};
  margin: ${({theme}) => theme.margin};
  background-color: ${({checked, theme}) => checked ? theme.color.primary : theme.color.background};
`;

export class Checkbox extends Component<any, any> {
  public static defaultProps = {
    onChange: () => null,
  };

  public state = {
    value: this.props.value || Boolean(this.props.defaultValue),
    prevValue: this.props.value || Boolean(this.props.defaultValue),
  };

  public static getDerivedStateFromProps(props, state) {
    const {value} = props;

    if (value !== state.prevValue) {
      return {
        value,
        prevValue: value,
      };
    }

    return null;
  }

  private onChange = (event) => {
    const {onChange, name} = this.props;
    const value = event.target.checked;

    this.setState({
      value,
    });

    onChange(value, name);
  };

  public render() {
    const {children, name, indeterminate} = this.props;
    const {value} = this.state;

    return (
      <ThemeConsumer component="checkbox">
        <Label checked={value}>
          <Row middle nowrap fluid>
            <Input name={name} onChange={this.onChange} checked={value}/>
            <Box checked={value || indeterminate}>
              <Icon icon={indeterminate && !value ? remove : tick} small primary/>
            </Box>
            {children}
          </Row>
        </Label>
      </ThemeConsumer>
    );
  }
}
