import React, {Component} from 'react';
import styled from 'styled-components';
import moment from 'moment';

const StyledPicker = styled.div`
  width: 252px;
  padding: 5px;
  margin: 0;
  font-size: 15px;
  border: 1px solid hsla(0, 0%, 0%, 0.25);
  border-radius: 4px;
  transition: all .1s;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  font-family: Tahoma,sans-serif;
  user-select: none;
`;

const PickerNav = styled.div`
  display: flex;
  justify-content: space-between;
  align-items: center;
  padding: 0 5px 5px;
  border-bottom: 1px solid hsla(0, 0%, 0%, 0.25);
`;

const Month = styled.div`
  width: 33%;
  height: 25%;
  display: flex;
  justify-content: center;
  align-items: center;
  cursor: pointer;
  background-color: ${(props) => props.selected ? '#5cc4ef' : ''};
  border-radius: 5px;
  color: ${(props) => props.selected ? '#fff' : props.current ? '#4EB5E6' : '#000'};
  &:hover {
    background-color: ${(props) => props.selected ? '#45bced' : '#F0F0F0'};
    border-radius: 5px;
  }
  &:focus {
    background-color: #5cc4ef;
  }
`;

const Svg = styled.svg`
  width: 32px;
  height: 32px;
  cursor: pointer;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const CurrentDate = styled.div`
  text-transform: capitalize;
  height: 32px;
  padding: 0 10px 0;
  cursor: pointer;
  display: flex;
  justify-content: center;
  align-items: center;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const Body = styled.div`
  margin-top: 5px;
  height: 170px;
  display: flex;
  flex-wrap: wrap;
  justify-content: space-between;
`;

export class MonthPicker extends Component<any, any> {
  public state = {
    date: this.props.date,
    months: [],
  };

  public componentDidMount = () => {
    const {date} = this.props;

    this.setState(() => {
      return {
        months: this.createMonths(date),
      };
    });
  };

  private chooseMonth = (event, month) => {
    event.preventDefault();

    const {date} = this.state;
    const {onSelect} = this.props;
    const newDate = date.clone().set('month', month);

    onSelect(newDate, date.year());

    this.setState(() => {
      return {
        date: newDate,
        months: this.createMonths(newDate),
      };
    });
  };

  private changeYear = (event, year) => {
    event.preventDefault();

    const {date} = this.state;
    const newDate = date.clone().set('year', year);

    this.setState(() => {
      return {
        date: newDate,
        months: this.createMonths(newDate),
      };
    });
  };

  public render() {
    const {date, months} = this.state;
    const {clickAtYears} = this.props;

    return (
      <StyledPicker>
        <PickerNav>
          <Svg onMouseDown={(event) => this.changeYear(event, date.clone().year() - 1)}>
            <path d="M 17,12 l -5,5 l 5,5"/>
          </Svg>
          <CurrentDate onClick={clickAtYears}>{date.format('YYYY')}</CurrentDate>
          <Svg onMouseDown={(event) => this.changeYear(event, date.clone().year() + 1)}>
            <path d="M 14,12 l 5,5 l -5,5"/>
          </Svg>
        </PickerNav>
        <Body>
        {months.map((item, index) => {
          const {month, key, active, name, current, selected} = item;

          return (
            <Month
              onMouseDown={(event) => this.chooseMonth(event, month)}
              key={key}
              active={active}
              current={current}
              selected={selected}
            >
              {name}
            </Month>
          );
        })}
        </Body>
      </StyledPicker>
    );
  }

  private createMonths(date) {
    const months = [];
    const {hasDisplayValue} = this.props;
    const {yearOfSelected} = this.props;
    const current = moment();

    for (let i = 0; i < 12; ++i) {
      months.push(
        {
          key: i,
          active: true,
          name: date.clone().set('month', i).format('MMMM'),
          month: i,
          current: i === current.month() && current.year() === date.year(),
          selected: hasDisplayValue ? i === date.month() && yearOfSelected === date.year() : false,
        });
    }

    return months;
  }
}
