import React, {Component} from 'react';
import styled from 'styled-components';
import moment from 'moment';

const StyledPicker = styled.div`
  width: 252px;
  padding: 5px;
  margin: 0;
  font-size: 15px;
  border: 1px solid hsla(0, 0%, 0%, 0.25);
  border-radius: 4px;
  transition: all .1s;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  font-family: Tahoma,sans-serif;
  user-select: none;
`;

const PickerNav = styled.div`
  display: flex;
  justify-content: space-between;
  align-items: center;
  padding: 0 5px 5px;
  border-bottom: 1px solid hsla(0, 0%, 0%, 0.25);
`;

const Year = styled.div`
  width: 33%;
  height: 25%;
  display: flex;
  justify-content: center;
  align-items: center;
  cursor: pointer;
  background-color: ${(props) => props.selected ? '#5cc4ef' : ''};
  border-radius: 5px;
  color: ${(props) => props.selected ? '#fff' : props.current ? '#4EB5E6' : props.active ? '#000' : '#dedede'};
  &:hover {
    background-color: ${(props) => props.selected ? '#45bced' : '#F0F0F0'};
    border-radius: 5px;
  }
  &:focus {
    background-color: #5cc4ef;
  }
`;

const Svg = styled.svg`
  width: 32px;
  height: 32px;
  cursor: pointer;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const CurrentDate = styled.div`
  text-transform: capitalize;
  height: 32px;
  padding: 0 10px 0;
  cursor: pointer;
  display: flex;
  justify-content: center;
  align-items: center;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const Body = styled.div`
  margin-top: 5px;
  height: 170px;
  display: flex;
  flex-wrap: wrap;
  justify-content: space-between;
`;

export class YearPicker extends Component<any, any> {
  public state = {
    date: this.props.date,
    yearOfSelected: this.props.date.year(),
    years: [],
    firstYear: 0,
    lastYear: 0,
  };

  public componentDidMount = () => {
    this.setState(() => {
      return {
        years: this.createYears(this.props.date, false),
        firstYear: +this.defineFirst(this.props.date.year()),
        lastYear: +this.defineFirst(this.props.date.year()) + 11,
      };
    });
  };

  private chooseYear = (event, year) => {
    event.preventDefault();

    const {date} = this.state;
    const {onSelect} = this.props;
    const newDate = date.clone().set('year', year);
    const yearsArray = this.createYears(newDate, true);

    onSelect(newDate);

    this.setState(() => {
      return {
        yearOfSelected: date.year(),
        date: newDate,
        years: yearsArray,
      };
    });
  };

  private changeYear = (event, year) => {
    event.preventDefault();

    const {date} = this.state;
    const {setYear} = this.props;
    const newDate = date.clone().set('year', year);
    const yearsArray = this.createYears(newDate, true);

    setYear(newDate);

    this.setState(() => {
      return {
        date: newDate,
        years: yearsArray,
      };
    });
  };

  public render() {
    const {date, years, firstYear, lastYear} = this.state;

    return (
      <StyledPicker>
        <PickerNav>
          <Svg onMouseDown={(event) => this.changeYear(event, date.clone().year() - 10)}>
            <path d="M 17,12 l -5,5 l 5,5"/>
          </Svg>
          <CurrentDate>{`${firstYear + 1} - ${lastYear - 1}`}</CurrentDate>
          <Svg onMouseDown={(event) => this.changeYear(event, date.clone().year() + 10)}>
            <path d="M 14,12 l 5,5 l -5,5"/>
          </Svg>
        </PickerNav>
        <Body>
        {years.map((item, index) => {
          const {year, key, active, name, current, selected} = item;

          return (
            <Year
              onMouseDown={(event) => this.chooseYear(event, year)}
              key={key}
              active={active}
              current={current}
              selected={selected}
            >
              {name}
            </Year>
          );
        })}
        </Body>
      </StyledPicker>
    );
  }

  private defineFirst = (year) => {
    if (year % 10 === 0) {
      return year - 1;
    } else {
      return year - year % 10 - 1;
    }
  };

  private createYears(date, setRange) {
    const years = [];
    const {hasDisplayValue} = this.props;
    const {yearOfSelected} = this.state;
    const firstYear = this.defineFirst(date.year());
    const lastYear = firstYear + 11;
    const current = moment();

    if (setRange) {
      this.setState(() => {
        return {
          firstYear,
          lastYear,
        };
      });
    }
    for (let i = firstYear; i <= lastYear; ++i) {
      years.push(
        {
          key: i,
          active: i !== firstYear && i !== lastYear,
          name: date.clone().set('year', i).format('YYYY'),
          year: i,
          current: current.year() === i,
          selected: hasDisplayValue ? yearOfSelected === i : false,
        });
    }

    return years;
  }
}
