import React, {Component} from 'react';
import styled from 'styled-components';

const FormContext = React.createContext({
  values: {},
  onChange: (name: string, value: any) => null,
});

export class Form extends Component<any, any> {
  public state = {
    values: this.props.values || {},
  };

  private promise: any = Promise.resolve();

  private onChange = (name: string, value: any) => {
    this.promise = this.promise.then(() => {
      return new Promise((resolve) => {
        const {values} = this.state;
        this.setState({
          values: {...values, [name]: value},
        }, () => {
          resolve();
        });
      });
    });
  };

  private onSubmit = (event) => {
    event.preventDefault();

    this.props.onSubmit(this.state.values);
    this.setState({values: {}});
  };

  public render() {
    const {children} = this.props;
    const {values} = this.state;

    return (
      <FormContext.Provider value={{values, onChange: this.onChange}}>
        <form action="/" method="POST" onSubmit={this.onSubmit}>
          {children}
        </form>
      </FormContext.Provider>
    );
  }
}

export const StyledFormField = styled.div`
  padding: 10px 0;
`;

export class FormField extends Component<any, any> {

  public componentDidMount() {
    const {defaultValue, name} = this.props;
    this.setDefault({defaultValue, name});
  }

  public componentWillReceiveProps(nextProps) {
    const {defaultValue, name} = nextProps;

    if (this.props.defaultValue !== nextProps.defaultValue) {
      this.setDefault({defaultValue, name});
    }
  }

  private setDefault({defaultValue, name}) {
    const {onChange} = (this as any).context;

    onChange(name, defaultValue);
  }

  public render() {
    const {component: Component, children, ...rest} = this.props;

    return (
      <StyledFormField>
        <FormContext.Consumer>
          {({values, onChange}) => {
            return Component ? (
              <Component
                {...rest}
                value={values[rest.name]}
                onChange={onChange.bind(null, rest.name)}
              >
                {children}
              </Component>
            ) : children;
          }}
        </FormContext.Consumer>
      </StyledFormField>
    );
  }
}

FormField.contextType = FormContext;
