import React from 'react';
import styled, {css} from 'styled-components';

const propsToColor = ({rating, primary, secondary, theme, color}) => {
  switch (true) {
    case Boolean(rating):
      return theme.color.ratingIcon;
    case Boolean(primary):
      return theme.color.primaryIcon;
    case Boolean(secondary):
      return theme.color.secondaryIcon;
    case Boolean(color):
      return color;
    default:
      return 'gray';
  }
};

const propsToSize = (({small, medium, defaultSize, originalSize}) => {
  switch (true) {
    case Boolean(small):
      return '20px';
    case Boolean(medium):
      return '25px';
    case Boolean(defaultSize && originalSize):
      return originalSize;
    default:
      return '42px';
  }
});

const svgDataFromJson = (svgChilds: any = []) => {
  return svgChilds.filter((child: any) => ['path', 'g', 'polygon', 'circle', 'rect'].includes(child.name))
    .map((element: any, key: number) => {
      return React.createElement(element.name, {...element.attrs, key}, svgDataFromJson(element.childs));
    });
};

export const Svg = styled.svg`
  width: ${(props) => propsToSize(props)};
  height: ${(props) => propsToSize(props)};
  fill: ${(props) => propsToColor(props)};
  ${({tick}) => tick ? css`
    position: absolute;
    z-index: 1;
  ` : null
  }
`;

export const Icon = ({icon, ...rest}) => {
  const {viewBox} = icon.attrs;
  let {width} = icon.attrs;

  if (!width.includes('px')) {
    width += 'px';
  }

  return (
    <Svg viewBox={viewBox} originalSize={width} {...rest} >
      {svgDataFromJson(icon.childs)}
    </Svg>
  );
};
