import React, {Component} from 'react';
import styled from 'styled-components';
import {Context} from './RadioGroup';

const StyledLabel = styled.label`
  display: flex;
  justify-content: flex-start;
  align-items: center;
  font-size: ${({theme}) => theme.font.control};
  cursor: pointer;
`;

const StyledInput = styled.input`
  appearance: none;
  border-radius: 50%;
  border: ${({theme}) => theme.color.primary} solid 3px;
  width: 25px;
  height: 25px;
  margin: 0 10px 0 0;
  padding: 0;
  cursor: pointer;
  :checked {
    background-color: ${({theme}) => theme.color.primary};
    box-shadow: inset 0 0 0 4px #fff;
  }
  :hover {
    background-color: ${({theme}) => theme.color.primary};
    box-shadow: inset 0 0 0 4px #fff;
  }
  :focus {
    outline: none;
  }
`;

export class Radio extends Component<any, any> {

  public static defaultProps = {
    onChange: (value) => null,
    value: null,
  };

  public render() {
    const {value: propsValue, children} = this.props;
    const stringifiedValue = JSON.stringify(propsValue);

    return (
      <Context.Consumer>
        {({name, onChange, value}) => (
          <StyledLabel>
            <StyledInput
              type="radio"
              value={stringifiedValue}
              name={name}
              onChange={onChange}
              checked={value === stringifiedValue}
            />
            {children}
          </StyledLabel>
        )}
      </Context.Consumer>
    );
  }
}
