import React, {Component} from 'react';
import styled from 'styled-components';

const Wrapper = styled.div`
  font-size: ${({theme}) => theme.font.control};
  justify-content: flex-start;
`;

export const Context = React.createContext({
  name: '',
  value: '',
  onChange: (value) => null,
});

export interface IRadioGroupProps {
  value: any;
  name: string;
  onChange: (value: any, name: string) => null;
}

export class RadioGroup extends Component<IRadioGroupProps, any> {

  public state = {
    prevValue: this.props.value || null,
    value: this.props.value || null,
  };

  public static defaultProps = {
    name: '',
    onChange: () => null,
  };

  public static getDerivedStateFromProps(props, state) {
    const {value} = props;
    const {prevValue} = state;

    if (value === prevValue) {
      return null;
    }

    return {
      prevValue: value,
      value,
    };
  }

  private onChange = (event) => {
    const {onChange, name} = this.props;
    const {value} = event.target;
    const parsedValue = JSON.parse(value);

    this.setState(() => ({
      value: parsedValue,
    }));

    onChange(parsedValue, name);
  };

  public render() {
    const {children, name} = this.props;
    const {value} = this.state;

    return (
      <Context.Provider
        value={{
          name,
          value: JSON.stringify(value),
          onChange: this.onChange,
        }}
      >
        <Wrapper>
          {children}
        </Wrapper>
      </Context.Provider>
    );
  }
}
