import React from 'react';
import styled from 'styled-components';
import {Text} from '../Text/Text';
import {Space} from '../Space/Space';
import {Icon} from '../Icon/Icon';
import star from './star.svg';

export interface IRateFieldProps {
  value?: number;
  onChange?(value: number);
}

export interface IRateFieldState {
  value: number;
}

const hints = {
  1: 'Very bad',
  2: 'Bad',
  3: 'Nothing special',
  4: 'Good',
  5: 'Great!',
};

const noop = (...args) => null;

const Button = styled.div`
  display: inline-block;
  cursor: pointer;
`;

export class RateField extends React.Component<IRateFieldProps, IRateFieldState> {

  public state = {
    value: this.props.value || 0,
  };

  private onClick = (value: number) => {
    const {onChange = noop} = this.props;
    this.setState({
      value,
    }, () => {
      onChange(value);
    });
  };

  public render() {
    const {value} = this.state;

    return (
      <div>
        <div>
          {[1, 2, 3, 4, 5].map((rate, key) => {
            const onClick = () => this.onClick(rate);
            return (
              <Button key={key} onClick={onClick}>
                <Icon rating={value >= rate} icon={star}/>
              </Button>
            );
          })}
        </div>
        <Space vertical tiny/>
        <Text><Space small/>{hints[value]}</Text>
      </div>
    );
  }
}
