import styled from 'styled-components';
import {fromRegistry, registry} from '../../core';
import {ITheme} from '../..';

const getColorByProps = (props, primaryDefault = false) => {
  switch (true) {
    case !!props.danger:
      return props.theme.color.danger;
    case !!props.primary:
      return !!props.theme.color.primary;
    case primaryDefault:
      return props.theme.color.text;
    default:
      return props.theme.color.textSecondary;
  }
};

export interface IText {
  block?: boolean;
  theme?: ITheme;
  primary?: boolean;
  danger?: boolean;
  bold?: boolean;
}

const TextComponent = styled.span`
  display: ${({block}) => block ? 'block' : 'inline'};
  font-family: ${(props) => props.theme.font.text.family};
  font-size: ${(props) => props.theme.font.text.size};
  color: ${(props) => getColorByProps(props)};
  font-weight: ${({bold}) => bold ? 'bold' : 'normal'};
  letter-spacing: normal;
  line-height: ${({theme}) => theme.font.lineHeight};
`;

registry.register('Text', TextComponent);

export const Text = fromRegistry<IText>('Text');

export const Label = styled(TextComponent)`
  font-size: 12px;
`;

const Header = styled(TextComponent)`
  display: block;
  color: ${(props) => getColorByProps(props, true)};
`;

export const H1 = styled(Header)`
  font-size: 40px;
`;

export const H2 = styled(Header)`
  font-size: 30px;
`;

export const H3 = styled(Header)`
  font-size: 20px;
`;
